# coding: utf-8
import os
import json
from odps.udf import annotate
from odps.udf import BaseUDTF
from odps.distcache import get_cache_file
from collections import defaultdict
os.environ['FG_VERSION'] = '${version}'
from load_pyfg import pyfg, include_file


def check_type(name, _type, value):
    _type = _type.lower()
    if _type in ('int', 'bigint') and not isinstance(value, int):
        print("  ->", name, ":", value, ", expect type:", _type, flush=True)
        return
    if _type in ('float', 'double') and not isinstance(value, float):
        print("  ->", name, ":", value, ", expect type:", _type, flush=True)
        return
    if _type == 'string' and not isinstance(value, str):
        print("  ->", name, ":", value, ", expect type:", _type, flush=True)
        return
    if _type.startswith('array'):
        if not isinstance(value, list):
            print("  ->", name, ":", value, ", expect type:", _type, flush=True)
            return
        sub_type = _type[6:-1]
        check_type(name + "_element", sub_type, value[0])
        return


@annotate('*->${output_types}')
class FgProcess(BaseUDTF):
    def __init__(self):
        self.reserve_columns = defaultdict(list)
        self.records = defaultdict(list)
        self.cur_index = 0
        config = self.load_config('${fg_json_file}')
        self.handler = pyfg.FgHandler(config)
        self.reserve_names = ${reserve_names}
        self.input_names = ${input_names}
        self.default_values = ${default_values}
        self.output_names = ${output_names}
        self.batch_size = ${batch_size}

    def load_config(self, fg_json):
        fg_file = get_cache_file(fg_json)
        config = json.load(fg_file)
        fg_file.close()
        assert "features" in config, "features not found in fg json config"
        cur_features = config['features']
        custom_ops = set()
        resource_paths = {}
        for sf in cur_features:
            for name in sf:
                if not str(name).endswith('_file'):
                    continue
                if sf[name].endswith('.so'):
                    custom_ops.add(sf[name])
                elif sf[name] in resource_paths:
                    sf[name] = resource_paths[sf[name]]
                else:
                    print("loading resource file:", sf[name], flush=True)
                    cur_file = get_cache_file(sf[name], 'b')
                    resource_paths[sf[name]] = cur_file.name
                    sf[name] = cur_file.name
                    cur_file.close()
        if custom_ops:
            pyfg.set_env('FEATURE_OPERATOR_DIR', os.getcwd())
            pyfg.set_env('FEATURE_OPERATOR_LIBS', ','.join(custom_ops))
            for so_file in custom_ops:
                print("loading so file:", so_file, flush=True)
                include_file(so_file, so_file)
        return config

    def output(self):
        if not self.records:
            return
        try:
            outputs, status = self.handler(self.records)
        except Exception as e:
            print("last input : ", self.records, flush=True)
            if isinstance(e, TypeError):
                for i in range(self.cur_index):
                    for rn in self.input_names:
                        cur_record = defaultdict(list)
                        for n, d in zip(self.input_names, self.default_values):
                            v = self.records[n][i]
                            if n != rn or v is None:
                                cur_record[n].append(d)
                            else:
                                cur_record[n].append(v)
                        try:
                            cur_outputs, status = self.handler(cur_record)
                            if not status.ok():
                                print("Feature : ", rn, ", not valid, status : ", status.message(), flush=True)
                        except Exception as se:
                            print("Feature : ", rn, ", not valid, value: ", cur_record[rn], flush=True)
                            print("record : ", cur_record, flush=True)
                            print("error : ", se, flush=True)
            raise e
        if not status.ok():
            print("status : ", status.message(), flush=True)
            if ${debug}:
                print("last input : ", self.records, flush=True)
                print("last output : ", outputs, flush=True)
            raise ValueError(status.message())
        for i in range(self.cur_index):
            return_list = []
            for col in self.reserve_names:
                return_list.append(self.reserve_columns[col][i])
            for col in self.output_names:
                if col in outputs:
                    return_list.append(outputs[col][i])
                else:
                    return_list.append(None)
                    if i == 0:
                        print("output column not found:", col, flush=True)
            try:
                self.forward(*return_list)
            except Exception as e:
                print("invalid output: ", e, flush=True)
                if len(${output_type_list}) != len(return_list):
                    print("invalid output length, expect", len(${output_type_list}), "got", len(return_list), flush=True)
                if isinstance(e, TypeError):
                    print("invalid output value type:")
                    for name, _type, value in zip(self.output_names, ${output_type_list}, return_list):
                        if name in self.reserve_names:
                            print("  reserved", name, ":", value)
                        check_type(name, _type, value)
                if ${debug}:
                    print("-------------------------------------------")
                    print("inputs:", self.records)
                    print("outputs:", outputs, flush=True)
                    print("forward:", return_list, flush=True)
                raise e
        self.records.clear()
        self.reserve_columns.clear()
        self.cur_index = 0

    def process(self, *record_values):
        for r, v in zip(self.reserve_names, record_values):
            self.reserve_columns[r].append(v)
        input_values = record_values[len(self.reserve_names):]
        for n, v, d in zip(self.input_names, input_values, self.default_values):
            self.records[n].append(d if v is None else v)
        self.cur_index += 1
        if self.cur_index == self.batch_size:
            self.output()

    def close(self):
        self.output()
 