# coding: utf-8
from odps.udf import annotate
from load_pyfg import pyfg

@annotate("string, string, string->double")
class ProximityFeature(object):
    def __init__(self):
        minCover = {
            "feature_type" : "overlap_feature",
            "feature_name" : "proximity_min_cover",
            "query" : "user:query",
            "title" : "item:title",
            "method" : "proximity_min_cover",
            "separator" : "\035"
        }
        self.minCoverOp = pyfg.OverlapFeature(minCover)

        minDist = {
            "feature_type" : "overlap_feature",
            "feature_name" : "proximity_min_dist",
            "query" : "user:query",
            "title" : "item:title",
            "method" : "proximity_min_dist",
            "separator" : "\035"
        }
        self.minDistOp = pyfg.OverlapFeature(minDist)

        maxDist = {
            "feature_type" : "overlap_feature",
            "feature_name" : "proximity_max_dist",
            "query" : "user:query",
            "title" : "item:title",
            "method" : "proximity_max_dist",
            "separator" : "\035"
        }
        self.maxDistOp = pyfg.OverlapFeature(maxDist)

        avgDist = {
            "feature_type" : "overlap_feature",
            "feature_name" : "proximity_avg_dist",
            "query" : "user:query",
            "title" : "item:title",
            "method" : "proximity_avg_dist",
            "separator" : "\035"
        }
        self.avgDistOp = pyfg.OverlapFeature(avgDist)

        queryRatio = {
            "feature_type" : "overlap_feature",
            "feature_name" : "query_common_ratio",
            "query" : "user:query",
            "title" : "item:title",
            "method" : "query_common_ratio",
            "separator" : "\035"
        }
        self.queryRatioOp = pyfg.OverlapFeature(queryRatio)

        titleRatio = {
            "feature_type" : "overlap_feature",
            "feature_name" : "title_common_ratio",
            "query" : "user:query",
            "title" : "item:title",
            "method" : "title_common_ratio",
            "separator" : "\035"
        }
        self.titleRatioOp = pyfg.OverlapFeature(titleRatio)
        print("FG init OK", flush=True)

    def evaluate(self, query, title, method):
        query = [query]
        title = [title]
        if method == 'min_cover':
            values = self.minCoverOp([query, title])
            return float(values[0])
        if method == 'min_dist':
            values = self.minDistOp([query, title])
            return float(values[0])
        if method == 'max_dist':
            values = self.maxDistOp([query, title])
            return float(values[0])
        if method == 'avg_dist':
            values = self.avgDistOp([query, title])
            return float(values[0])
        if method == 'query_ratio':
            values = self.queryRatioOp([query, title])
            return float(values[0])
        if method == 'title_ratio':
            values = self.titleRatioOp([query, title])
            return float(values[0])
        raise ValueError('unsupported method: ' + method)
